import { SkillPriority } from './models';
import { RestClient } from './RestClient';
export declare class AgentClient {
    private restClient;
    private subAccountId;
    private subAccountAppId;
    private defaultSkillNumber;
    constructor(subAccountId: string, subAccountAppId: string, restClient: RestClient);
    get SubAccountAppId(): string;
    /**
     * @protected
     */
    getDefaultSkillNumber(): number;
    /**
     * Create Agent and Station. Upon success, returns agent object and station object
     * @param agentUsername min length 2, max length 20, must pass ^[-.@\w]+$
     * @param agentPassword min length 8, max length 32, must have a uppercase character, must have at least one lowercase char, no whitespace, must contains a number, must contain one of ~!@?#$%^&*_
     * @param firstName (optional) min length 2, max 16, must pass /^[a-zA-Z][a-zA-Z0-9_ .,'+]*$/
     * @param lastName (optional) min length 2, max 16, must pass /^[a-zA-Z][a-zA-Z0-9_ .,'+]*$/
     */
    createAgentAndStation(agentUsername: string, agentPassword: string, firstName?: string, lastName?: string): Promise<{
        agent: any;
        station: any;
    }>;
    /**
     * @protected
     */
    setDefaultSkillNumberIfNotExists(): Promise<boolean>;
    /**
     * @protected
     */
    createUserIfNotExists(agentUsername: string, agentPassword: string, skillsWithPriority: SkillPriority[], agentStationGroupId: string, firstName?: string, lastName?: string): Promise<boolean>;
    /**
     * @protected
     */
    sendCreateAgentRequest(agentUsername: string, agentPassword: string, agentStationGroupId: any, agentLoginId: any, skillsWithPriority: SkillPriority[], firstname?: string, lastname?: string): Promise<any>;
    /**
     * @protected
     */
    generateAvayaPassword(agentLoginId: {
        toString: () => any;
    }): any;
    /**
     * @protected
     */
    generateSecurityCode(agentLoginId: {
        toString: () => any;
    }): any;
    /**
     * @protected
     */
    getSkillIds(): Promise<[]>;
    /**
     * @protected
     */
    fetchDefaultSkillNumber(): Promise<number | undefined>;
    /**
     * @protected
     */
    isDefaultSkillProvisioned(skillNumber: number): Promise<boolean>;
    /**
     * @protected
     */
    createDefaultSkill(): Promise<number>;
    /**
     * retrieve agent skills in {skillName:string, skillNumber:number}[]
     * @protected
     */
    getSkillNumbers(): Promise<Array<{
        skillName: string;
        skillNumber: number;
    }>>;
    /**
     * @protected
     */
    getAgent(agentUsername: string): Promise<any>;
    /**
     * @protected
     */
    getAgentAndStation(agentUsername: string): Promise<{
        agent: any;
        station: any;
    }>;
    /**
     * @protected
     */
    existsAgentByLoginId(loginId: number): Promise<boolean>;
    /**
     * @protected
     */
    existsAgentByUsername(username: string): Promise<boolean>;
    /**
     * @protected
     */
    existsAgent(promise: Promise<any>): Promise<boolean>;
    /**
     * redo action, as a callback function, a number of times
     * @param callback need to always resolve to either true or false
     * @param retries max number of retries
     * @param millis time to sleep before retry
     * @protected
     */
    redo(callback: () => Promise<boolean>, retries: number, millis: number): Promise<boolean>;
    /**
     * @protected
     */
    waitForAgentCreation(loginId: number): Promise<boolean>;
    /**
     * @protected
     */
    waitForAgentDeletion(agentUsername: string): Promise<boolean>;
    /**
     * @protected
     */
    waitForDefaultSkillCreation(skillNumber: number): Promise<boolean>;
    /**
     * delete agent and station
     * @param agentUsername username of agent
     */
    deleteAgentAndStation(agentUsername: string): Promise<boolean>;
    getUserToken(username: string): Promise<any>;
    private repeat;
}
export declare function createAgentClient(endpoint: string, apiKey: string): Promise<AgentClient>;
