"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createAgentClient = exports.AgentClient = void 0;
const Constants = __importStar(require("./Constants"));
const RestClient_1 = require("./RestClient");
const Utils_1 = require("./Utils");
const logger = Constants.log4js.getLogger('AgentClient');
class AgentClient {
    constructor(subAccountId, subAccountAppId, restClient) {
        this.restClient = restClient;
        this.subAccountId = subAccountId;
        this.subAccountAppId = subAccountAppId;
        this.defaultSkillNumber = -1;
        logger.debug(`subAccountAppid=${subAccountAppId}; accountId = ${subAccountId}`);
    }
    get SubAccountAppId() {
        return this.subAccountAppId;
    }
    /**
     * @protected
     */
    getDefaultSkillNumber() {
        return this.defaultSkillNumber;
    }
    /**
     * Create Agent and Station. Upon success, returns agent object and station object
     * @param agentUsername min length 2, max length 20, must pass ^[-.@\w]+$
     * @param agentPassword min length 8, max length 32, must have a uppercase character, must have at least one lowercase char, no whitespace, must contains a number, must contain one of ~!@?#$%^&*_
     * @param firstName (optional) min length 2, max 16, must pass /^[a-zA-Z][a-zA-Z0-9_ .,'+]*$/
     * @param lastName (optional) min length 2, max 16, must pass /^[a-zA-Z][a-zA-Z0-9_ .,'+]*$/
     */
    createAgentAndStation(agentUsername, agentPassword, firstName, lastName) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!Utils_1.isValidPassword(agentPassword)) {
                return Promise.reject('invalid password');
            }
            if (!Utils_1.isValidUsername(agentUsername)) {
                return Promise.reject('invalid username');
            }
            if (firstName && !Utils_1.isValidFirstLastName(firstName)) {
                return Promise.reject('invalid first name');
            }
            if (lastName && !Utils_1.isValidFirstLastName(lastName)) {
                return Promise.reject('invalid last name');
            }
            if (this.defaultSkillNumber === -1) {
                const success = yield this.setDefaultSkillNumberIfNotExists();
                if (!success) {
                    return Promise.reject('Can not create default skill for agent creation.');
                }
            }
            const skillsWithPriority = [
                {
                    skillNumber: this.defaultSkillNumber,
                    skillPriority: Constants.DEFAULT_SKILL_PRIORITY
                }
            ];
            const agentStationGroupId = yield this.restClient.getAgentStationGroupId(this.subAccountAppId);
            if (agentStationGroupId < 0) {
                throw new Error(`subAccount ${this.subAccountId} has no agent station group defined`);
            }
            yield this.createUserIfNotExists(agentUsername, agentPassword, skillsWithPriority, agentStationGroupId, firstName, lastName);
            return this.getAgentAndStation(agentUsername);
        });
    }
    /**
     * @protected
     */
    setDefaultSkillNumberIfNotExists() {
        return __awaiter(this, void 0, void 0, function* () {
            const defaultSkillNumber = yield this.fetchDefaultSkillNumber();
            logger.debug(`defaultSkillNumber is ${defaultSkillNumber}`);
            if (!!defaultSkillNumber) {
                this.defaultSkillNumber = defaultSkillNumber;
                return true;
            }
            else {
                logger.debug('tring to create default skill');
                const skillNumber = yield this.createDefaultSkill();
                this.defaultSkillNumber = skillNumber;
                return yield this.waitForDefaultSkillCreation(skillNumber);
            }
        });
    }
    /**
     * @protected
     */
    createUserIfNotExists(agentUsername, agentPassword, skillsWithPriority, agentStationGroupId, firstName, lastName) {
        return __awaiter(this, void 0, void 0, function* () {
            const userExists = yield this.existsAgentByUsername(agentUsername);
            if (userExists) {
                return Promise.resolve(true);
            }
            const agentLoginId = yield this.restClient.getNextAvailableExtension(this.subAccountId, 'AGENT');
            if (agentLoginId < 0) {
                throw new Error(`subAccount ${this.subAccountId} has no available agent extension`);
            }
            yield this.sendCreateAgentRequest(agentUsername, agentPassword, agentStationGroupId, agentLoginId, skillsWithPriority, firstName, lastName);
            return yield this.waitForAgentCreation(agentLoginId);
        });
    }
    /**
     * @protected
     */
    sendCreateAgentRequest(agentUsername, agentPassword, agentStationGroupId, agentLoginId, skillsWithPriority, firstname, lastname) {
        return __awaiter(this, void 0, void 0, function* () {
            const securityCode = this.generateSecurityCode(agentLoginId);
            const avayaPassword = this.generateAvayaPassword(agentLoginId);
            if (!firstname || !lastname) {
                firstname = Constants.AGENT_FIRST_NAME;
                lastname = Constants.AGENT_LAST_NAME;
            }
            const agent = {
                username: agentUsername,
                firstName: firstname,
                lastName: lastname,
                password: agentPassword,
                loginId: agentLoginId,
                agentStationGroupId,
                securityCode,
                startDate: '2019-03-21',
                endDate: '2038-01-01',
                avayaPassword,
                clientId: this.subAccountId,
                agentSkills: skillsWithPriority,
                // no supervisors
                supervisorId: 0,
                // channel 1 is voice
                channelIds: [1]
            };
            return this.restClient.createAgentJob(agent);
        });
    }
    /**
     * @protected
     */
    generateAvayaPassword(agentLoginId) {
        const agentLoginIdString = agentLoginId.toString();
        const length = agentLoginIdString.length;
        // substring(starting_index, ending_index), negative starting_index is treated as 0
        return agentLoginIdString.substring(length - 6, length);
    }
    /**
     * @protected
     */
    generateSecurityCode(agentLoginId) {
        const agentLoginIdString = agentLoginId.toString();
        const length = agentLoginIdString.length;
        // substring(starting_index, ending_index), negative starting_index is treated as 0
        return agentLoginIdString.substring(length - 4, length);
    }
    /**
     * @protected
     */
    getSkillIds() {
        return this.restClient
            .getSubAccountAgentSkills(this.subAccountId)
            .then((skillResponses) => {
            logger.debug(skillResponses);
            if (skillResponses) {
                return skillResponses.map((skillResponse) => skillResponse.id);
            }
            else {
                return [];
            }
        });
    }
    /**
     * @protected
     */
    fetchDefaultSkillNumber() {
        return this.restClient
            .getSubAccountAgentSkills(this.subAccountAppId)
            .then((response) => {
            logger.debug(response.data);
            const defaultSkill = response.data.find((skillResponse) => skillResponse.name === Constants.DEFAULT_SKILL_NAME);
            return defaultSkill === null || defaultSkill === void 0 ? void 0 : defaultSkill.skillNumber;
        });
    }
    /**
     * @protected
     */
    isDefaultSkillProvisioned(skillNumber) {
        return this.restClient.getSkillV2(skillNumber).then((response) => {
            logger.debug(response.data.status);
            return response.data.status === 'COMPLETED';
        });
    }
    /**
     * @protected
     */
    createDefaultSkill() {
        return __awaiter(this, void 0, void 0, function* () {
            const skillExtensionNumber = yield this.restClient.getNextAvailableNumber(this.subAccountId, 'SKILL');
            if (skillExtensionNumber < 0) {
                return Promise.reject(false);
            }
            const skillRequest = {
                name: Constants.DEFAULT_SKILL_NAME,
                skillNumber: skillExtensionNumber,
                subAccountAppId: this.subAccountAppId,
                acwInterval: null,
                slaInSeconds: null,
                slaPercentage: null,
                announcementExtension: null,
                redirectOnNoAnswerRings: null,
                chatDigitalChannelsEnabled: false,
                emailDigitalChannelsEnabled: false,
                smsDigitalChannelsEnabled: false,
                dispositionSetId: ''
            };
            logger.debug(`creating default skill with payload ${JSON.stringify(skillRequest)}`);
            const result = yield this.restClient.createSkillV2(skillRequest);
            if (result === 200) {
                return skillExtensionNumber;
            }
            throw new Error(`Failed to create default skill error is ${-result}`);
        });
    }
    /**
     * retrieve agent skills in {skillName:string, skillNumber:number}[]
     * @protected
     */
    getSkillNumbers() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.restClient
                .getSubAccountAgentSkills(this.subAccountAppId)
                .then((skillResponses) => {
                const availableSkills = [];
                for (const skill of skillResponses) {
                    const skillInfo = {
                        skillNumber: skill.number,
                        skillName: skill.name
                    };
                    availableSkills.push(skillInfo);
                }
                logger.debug(availableSkills);
                return availableSkills;
            });
        });
    }
    /**
     * @protected
     */
    getAgent(agentUsername) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.restClient
                .getAgentByUsername(agentUsername)
                .then(agent => agent)
                .catch(error => undefined);
        });
    }
    /**
     * @protected
     */
    getAgentAndStation(agentUsername) {
        return __awaiter(this, void 0, void 0, function* () {
            const agent = yield this.getAgent(agentUsername);
            // for backwards-compatibility
            const station = agent.station;
            return { agent: agent || {}, station: station || {} };
        });
    }
    /**
     * @protected
     */
    existsAgentByLoginId(loginId) {
        const promise = this.restClient.getAgentByLoginId(loginId);
        return this.existsAgent(promise);
    }
    /**
     * @protected
     */
    existsAgentByUsername(username) {
        const promise = this.restClient.getAgentByUsername(username);
        return this.existsAgent(promise);
    }
    /**
     * @protected
     */
    existsAgent(promise) {
        return promise
            .then(agent => {
            return true;
        })
            .catch(error => {
            return false;
        });
    }
    /**
     * redo action, as a callback function, a number of times
     * @param callback need to always resolve to either true or false
     * @param retries max number of retries
     * @param millis time to sleep before retry
     * @protected
     */
    redo(callback, retries, millis) {
        return __awaiter(this, void 0, void 0, function* () {
            logger.debug(`entering redo ${retries}`);
            for (let count = 0; count < retries; count++) {
                const result = yield callback();
                logger.debug(`redo count=${count}; result=${result}`);
                if (result) {
                    logger.debug('result is true exiting redo...');
                    return true;
                }
                yield Utils_1.sleep(millis);
            }
            return false;
        });
    }
    /**
     * @protected
     */
    waitForAgentCreation(loginId) {
        return __awaiter(this, void 0, void 0, function* () {
            const callback = () => {
                return this.existsAgentByLoginId(loginId);
            };
            return this.repeat(callback);
        });
    }
    /**
     * @protected
     */
    waitForAgentDeletion(agentUsername) {
        return __awaiter(this, void 0, void 0, function* () {
            const callback = () => {
                return this.existsAgentByUsername(agentUsername).then(result => !result);
            };
            return this.repeat(callback);
        });
    }
    /**
     * @protected
     */
    waitForDefaultSkillCreation(skillNumber) {
        return __awaiter(this, void 0, void 0, function* () {
            const callback = () => {
                return this.isDefaultSkillProvisioned(skillNumber);
            };
            return this.repeat(callback);
        });
    }
    /**
     * delete agent and station
     * @param agentUsername username of agent
     */
    deleteAgentAndStation(agentUsername) {
        return __awaiter(this, void 0, void 0, function* () {
            const agent = yield this.getAgent(agentUsername);
            if (!agent) {
                return true;
            }
            const submitted = yield this.restClient.requestAgentDeletion(agentUsername, agent.loginId);
            if (submitted) {
                return yield this.waitForAgentDeletion(agentUsername);
            }
            return false;
        });
    }
    getUserToken(username) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.restClient.getUserToken(username);
        });
    }
    repeat(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.redo(callback, Constants.MAX_RETRY, Constants.INTERVAL_IN_MILLIS);
        });
    }
}
exports.AgentClient = AgentClient;
/**
 * @protected
 */
function createInstance(restClient) {
    return __awaiter(this, void 0, void 0, function* () {
        const result = yield restClient.getSubAccountIdAndAppId();
        return new AgentClient(result.id, result.appId, restClient);
    });
}
function createAgentClient(endpoint, apiKey) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!Utils_1.isValidUrl(endpoint)) {
            return Promise.reject('invalid endpoint');
        }
        if (!Utils_1.isTokenWellFormed(apiKey)) {
            return Promise.reject('invalid api key');
        }
        const restClient = new RestClient_1.RestClient(endpoint, apiKey);
        return yield createInstance(restClient);
    });
}
exports.createAgentClient = createAgentClient;
//# sourceMappingURL=AgentClient.js.map