/**
 * @hidden
 * @ignore
 * @internal
 */
/**
 * This comment and the above tags will tell typedoc to ignore the whole module.
 */
import { CookieJar } from 'tough-cookie';
import { SkillCreateRequest } from './models';
import { AddressBookSearchRequest } from './models';
export declare const STATION_GROUP_ID_NOT_EXISTS = -1;
interface Credential {
    token: string;
    cookieJar: CookieJar;
}
export declare class RestClient {
    masterCredential: Credential;
    credentials: Map<string, Credential>;
    private baseUrl;
    constructor(baseUrl: string, masterToken: string);
    getUser(): any;
    getUserToken(username: string): any;
    storeUserToken(username: string, userToken: string): void;
    getAndStoreUserToken(username: string): Promise<void>;
    /**
     * Caller should await the method to finish. When call is successful, a number greater than 0 will be returned.
     * When subAccountId has no station group defined, -1 will be returned;
     * For other errors, a negative value of http status code will be returned;
     * @param subAccountId
     */
    getAgentStationGroupId(subAccountAppId: string): any;
    getSubAccount(): any;
    getSubAccountId(): Promise<any>;
    getSubAccountAppId(): Promise<any>;
    getSubAccountIdAndAppId(): Promise<any>;
    getAgentByUsername(agentUsername: string): Promise<any>;
    /**
     * return true if agent deletion is requested successfully
     * return false otherwise
     * @param agentUsername
     * @param agentLoginId
     */
    requestAgentDeletion(agentUsername: string, agentLoginId: any): any;
    prepareBaseOptions(): {
        headers: {
            Authorization: string;
        };
        jar: CookieJar;
        withCredentials: boolean;
    };
    prepareGetOptions(url: string): {
        url: string;
        method: string;
        headers: {
            Authorization: string;
        };
        jar: CookieJar;
        withCredentials: boolean;
    };
    preparePostOptions(url: string): {
        url: string;
        method: string;
        headers: {
            Authorization: string;
        };
        jar: CookieJar;
        withCredentials: boolean;
    };
    prepareDeleteOptions(url: string): {
        url: string;
        method: string;
        headers: {
            Authorization: string;
        };
        jar: CookieJar;
        withCredentials: boolean;
    };
    /**
     * return agent or undefined
     * @param loginId
     */
    getAgentByLoginId(loginId: number): Promise<any>;
    createAgentJob(agent: any): any;
    /**
     * return extension number if successful.otherwise a negative status code
     * @param subAccountId
     * @param type
     */
    getNextAvailableExtension(subAccountId: string, type: string): Promise<number>;
    getNextAvailableNumber(subAccountId: string, type: string): Promise<number>;
    createSkillV2(skill: SkillCreateRequest): Promise<number>;
    getSubAccountAgentSkills(subAccountAppId: string): Promise<any>;
    getSkillV2(skillNumber: number): Promise<boolean>;
    makeSubAccountSubscriptionUrl(subAccountAppId: string | undefined): string;
    /**
     * create data subscription given a valid subAccountAppId and request.  Returns a subscription response on success
     * or empty object on error
     *
     * @param createSubscriptionRequest
     */
    createDataSubscription(createSubscriptionRequest: any): any;
    getAllSubscriptions(subAccountAppId: string): any;
    /**
     *
     * @param subAccountAppId
     * @param subscriptionId
     * @param updateSubscriptionRequest
     */
    updateDataSubscription(subAccountAppId: string, subscriptionId: any, updateSubscriptionRequest: any): any;
    /**
     * return status code when success or negative status code in error
     * @param subAccountAppId
     * @param subscriptionId usually it is a uuid
     */
    deleteDataSubscription(subAccountAppId: string, subscriptionId: string): number;
    makeSubscriptionUrl(subAccountAppId: string, subscriptionId: string): string;
    /**
     * get Subscription by id
     * @param subAccountAppId
     * @param subscriptionId
     */
    getDataSubscription(subAccountAppId: string, subscriptionId: string): any;
    printMasterCookieJar(): string;
    getAuxCodesBySubaccountId(subAccountId: string): Promise<any>;
    getEffectiveAuxCodesBySubaccountId(subAccountId: string): Promise<any>;
    getAUXCodeForEffectiveAppId(): Promise<any>;
    getAddressBook(): Promise<any>;
    searchContacts(searchRequest?: AddressBookSearchRequest): Promise<any>;
    protected postToUrl(url: string): Promise<number>;
}
export {};
