"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSubscriptionClient = exports.SubscriptionClient = void 0;
const Constants_1 = require("./Constants");
const RestClient_1 = require("./RestClient");
const logger = Constants_1.log4js.getLogger('SubscriptionClient');
class SubscriptionClient {
    constructor(subAccountAppId, restClient) {
        this.subAccountAppId = subAccountAppId;
        this.restClient = restClient;
    }
    /**
     * create a data subscription on the default sub account and returns a Subscription object
     * @param createSubscriptionRequest
     */
    createSubscription(createSubscriptionRequest) {
        return __awaiter(this, void 0, void 0, function* () {
            const returnedSubscription = yield this.restClient
                .createDataSubscription(Object.assign({}, createSubscriptionRequest, {
                subAccountAppId: this.subAccountAppId
            }))
                .then((result) => result);
            return returnedSubscription;
        });
    }
    /**
     * return all subscripitions of the default sub account
     */
    getAllSubscriptions() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.restClient.getAllSubscriptions(this.subAccountAppId);
        });
    }
    /**
     * Update subscription and returns updated Subscription object
     * @param updateSubscriptionRequest
     */
    updateSubscription(updateSubscriptionRequest) {
        return __awaiter(this, void 0, void 0, function* () {
            updateSubscriptionRequest.subAccountAppId = this.subAccountAppId;
            const returnedSubscription = yield this.restClient
                .updateDataSubscription(this.subAccountAppId, updateSubscriptionRequest.subscriptionId, updateSubscriptionRequest)
                .then((result) => result);
            return returnedSubscription;
        });
    }
    /**
     * Delete subscription by subscription id. Returns 200 if successful, negative status code in error.
     * @param subscriptionId
     */
    deleteSubscription(subscriptionId) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.restClient.deleteDataSubscription(this.subAccountAppId, subscriptionId);
        });
    }
    /**
     * returns Subscription object given subscription id or an empty object
     * @param subscriptionId
     */
    getSubscription(subscriptionId) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.restClient.getDataSubscription(this.subAccountAppId, subscriptionId);
        });
    }
}
exports.SubscriptionClient = SubscriptionClient;
function createInstance(restClient) {
    return __awaiter(this, void 0, void 0, function* () {
        const subAccountAppId = yield restClient.getSubAccountAppId();
        return new SubscriptionClient(subAccountAppId, restClient);
    });
}
/**
 * create SubscriptionClient given endpoint and apiKey. Note to user: SubAccountAppId is used interally and the only
 * allowable feed will be REAL_TIME_FEED.
 * @param endpoint
 * @param apiKey
 */
function createSubscriptionClient(endpoint, apiKey) {
    return __awaiter(this, void 0, void 0, function* () {
        const restClient = new RestClient_1.RestClient(endpoint, apiKey);
        return yield createInstance(restClient);
    });
}
exports.createSubscriptionClient = createSubscriptionClient;
//# sourceMappingURL=SubscriptionClient.js.map